@@ -1,4 +1,5 @@
 import contextlib
+import glob
 import io
 import json
 import os
@@ -297,7 +298,7 @@ DEFAULT_INGESTION_TYPE = 'metadata_bundle'
 
 
 def submit_any_ingestion(ingestion_filename, ingestion_type, institution, project, server, env, validate_only,
-                         upload_folder=None):
+                         upload_folder=None, no_query=False, subfolders=False):
     """
     Does the core action of submitting a metadata bundle.
 
@@ -309,6 +310,8 @@ def submit_any_ingestion(ingestion_filename, ingestion_type, institution, projec
     :param env: the beanstalk environment to upload to
     :param validate_only: whether to do stop after validation instead of proceeding to post metadata
     :param upload_folder: folder in which to find files to upload (default: same as bundle_filename)
+    :param no_query: bool to suppress requests for user input
+    :param subfolders: bool to search subdirectories within upload_folder for files
     """
 
     with script_catch_errors():
@@ -321,10 +324,11 @@ def submit_any_ingestion(ingestion_filename, ingestion_type, institution, projec
         if ingestion_type != DEFAULT_INGESTION_TYPE:
             maybe_ingestion_type = " (%s)" % ingestion_type
 
-        if not yes_or_no("Submit %s%s to %s%s?"
-                         % (ingestion_filename, maybe_ingestion_type, server, validation_qualifier)):
-            show("Aborting submission.")
-            exit(1)
+        if not no_query:
+            if not yes_or_no("Submit %s%s to %s%s?"
+                             % (ingestion_filename, maybe_ingestion_type, server, validation_qualifier)):
+                show("Aborting submission.")
+                exit(1)
 
         keydict = get_keydict_for_server(server)
         keypair = keydict_to_keypair(keydict)
@@ -430,7 +434,9 @@ def submit_any_ingestion(ingestion_filename, ingestion_type, institution, projec
 
         if outcome == 'success':
             show_section(res, 'upload_info')
-            do_any_uploads(res, keydict=keydict, ingestion_filename=ingestion_filename, upload_folder=upload_folder)
+            do_any_uploads(res, keydict=keydict, ingestion_filename=ingestion_filename,
+                           upload_folder=upload_folder, no_query=no_query,
+                           subfolders=subfolders)
 
         exit(0)
 
@@ -460,17 +466,24 @@ def show_upload_info(uuid, server=None, env=None, keydict=None):
             show("No uploads.")
 
 
-def do_any_uploads(res, keydict, upload_folder=None, ingestion_filename=None):
+def do_any_uploads(res, keydict, upload_folder=None, ingestion_filename=None,
+                   no_query=False, subfolders=False):
     upload_info = get_section(res, 'upload_info')
+    folder = upload_folder or (os.path.dirname(ingestion_filename) if ingestion_filename else None)
     if upload_info:
-        if yes_or_no("Upload %s?" % n_of(len(upload_info), "file")):
-            do_uploads(upload_info, auth=keydict,
-                       folder=upload_folder or (os.path.dirname(ingestion_filename) if ingestion_filename else None))
+        if no_query:
+            do_uploads(upload_info, auth=keydict, no_query=no_query, folder=folder,
+                       subfolders=subfolders)
         else:
-            show("No uploads attempted.")
+            if yes_or_no("Upload %s?" % n_of(len(upload_info), "file")):
+                do_uploads(upload_info, auth=keydict, no_query=no_query, folder=folder,
+                           subfolders=subfolders)
+            else:
+                show("No uploads attempted.")
 
 
-def resume_uploads(uuid, server=None, env=None, bundle_filename=None, keydict=None, upload_folder=None):
+def resume_uploads(uuid, server=None, env=None, bundle_filename=None, keydict=None,
+                   upload_folder=None, no_query=False, subfolders=False):
     """
     Uploads the files associated with a given ingestion submission. This is useful if you answered "no" to the query
     about uploading your data and then later are ready to do that upload.
@@ -481,6 +494,8 @@ def resume_uploads(uuid, server=None, env=None, bundle_filename=None, keydict=No
     :param bundle_filename: the bundle file to be uploaded
     :param keydict: keydict-style auth, a dictionary of 'key', 'secret', and 'server'
     :param upload_folder: folder in which to find files to upload (default: same as ingestion_filename)
+    :param no_query: bool to suppress requests for user input
+    :param subfolders: bool to search subdirectories within upload_folder for files
     """
 
     with script_catch_errors():
@@ -494,7 +509,9 @@ def resume_uploads(uuid, server=None, env=None, bundle_filename=None, keydict=No
         do_any_uploads(response.json(),
                        keydict=keydict,
                        ingestion_filename=bundle_filename,
-                       upload_folder=upload_folder)
+                       upload_folder=upload_folder,
+                       no_query=no_query,
+                       subfolders=subfolders)
 
 
 def execute_prearranged_upload(path, upload_credentials):
@@ -555,7 +572,7 @@ def upload_file_to_uuid(filename, uuid, auth):
 CGAP_SELECTIVE_UPLOADS = environ_bool("CGAP_SELECTIVE_UPLOADS")
 
 
-def do_uploads(upload_spec_list, auth, folder=None):
+def do_uploads(upload_spec_list, auth, folder=None, no_query=False, subfolders=False):
     """
     Uploads the files mentioned in the give upload_spec_list.
 
@@ -564,14 +581,32 @@ def do_uploads(upload_spec_list, auth, folder=None):
     :param auth: a dictionary-form auth spec, of the form {'key': ..., 'secret': ..., 'server': ...}
         representing destination and credentials.
     :param folder: a string naming a folder in which to find the filenames to be uploaded.
+    :param no_query: bool to suppress requests for user input
+    :param subfolders: bool to search subdirectories within upload_folder for files
     :return: None
     """
+    folder = folder or os.path.curdir
+    if subfolders:
+        folder = os.path.join(folder, '**')
     for upload_spec in upload_spec_list:
-        filename = os.path.join(folder or os.path.curdir, upload_spec['filename'])
-        uuid = upload_spec['uuid']
-        if CGAP_SELECTIVE_UPLOADS and not yes_or_no("Upload %s?" % (filename,)):
-            show("OK, not uploading it.")
+        file_path = os.path.join(folder, upload_spec['filename'])
+        file_search = glob.glob(file_path, recursive=subfolders)
+        if len(file_search) == 1:
+            filename = file_search[0]
+        elif len(file_search) > 1:
+            show(
+                "No upload attempted for file %s because multiple copies were found"
+                " in folder %s: %s."
+                % (upload_spec['filename'], folder, ", ".join(file_search))
+            )
             continue
+        else:
+            filename = file_path
+        uuid = upload_spec['uuid']
+        if not no_query:
+            if CGAP_SELECTIVE_UPLOADS and not yes_or_no("Upload %s?" % (filename,)):
+                show("OK, not uploading it.")
+                continue
         try:
             show("Uploading %s to item %s ..." % (filename, uuid))
             upload_file_to_uuid(filename=filename, uuid=uuid, auth=auth)
@@ -580,7 +615,7 @@ def do_uploads(upload_spec_list, auth, folder=None):
             show("%s: %s" % (e.__class__.__name__, e))
 
 
-def upload_item_data(item_filename, uuid, server, env):
+def upload_item_data(item_filename, uuid, server, env, no_query=False):
     """
     Given a part_filename, uploads that filename to the Item specified by uuid on the given server.
 
@@ -590,6 +625,7 @@ def upload_item_data(item_filename, uuid, server, env):
     :param uuid: the UUID of the Item with which the uploaded data is to be associated
     :param server: the server to upload to (where the Item is defined)
     :param env: the beanstalk environment to upload to (where the Item is defined)
+    :param no_query: bool to suppress requests for user input
     :return:
     """
 
@@ -599,8 +635,9 @@ def upload_item_data(item_filename, uuid, server, env):
 
     # print("keydict=", json.dumps(keydict, indent=2))
 
-    if not yes_or_no("Upload %s to %s?" % (item_filename, server)):
-        show("Aborting submission.")
-        exit(1)
+    if not no_query:
+        if not yes_or_no("Upload %s to %s?" % (item_filename, server)):
+            show("Aborting submission.")
+            exit(1)
 
     upload_file_to_uuid(filename=item_filename, uuid=uuid, auth=keydict)
